{% extends 'layouts/main.html' %}
{% block title %}Monitor an Experiment{% endblock %}
{% block content %}

<div class="page-header">
  <h1>This is the output of your subprocess; LIVE!</h1>
</div>
<div>
<p class="monospace">
<iframe
src="{{ url_for('generate_stdout') }}"
width="100%"
height="500"
allowfullscreen=true
title="Subprocess output"
style="border:none;"
>
</iframe>
</p>
</div>
{% endblock %}
