{% extends 'layouts/main.html' %}
{% block title %}Join Tables{% endblock %}
{% block content %}

<div class="page-header">
    {% from 'macros/render_form.html' import joindownload_render_form %}
    <h2> <a href={{url}}>Join/Download Tables</a> <h2>
    <h6>
        Join arbitrary tables that exist in the database, or download a
        pandas pickle file of the joined table.
        It is not required to add a restriction, although a large table
        especially with blobs (i.e. data or attachments) can take a long
        time to download. You can also just join/download
        a single table.
    </h6>
    {{ joindownload_render_form(form) }}
    <hr>
    <h4>Joined Table</h4>
    {% with data=data, toggle_off_keys=toggle_off_keys%}
        {% include 'macros/tables.html' %}
    {% endwith %}
</div>

{% endblock %}
