{% extends 'layouts/main.html' %}
{% block title %}Home{% endblock %}
{% block content %}

<div class="page-header">
  <h1>Hello {{user}}! <font color="crimson">Stay at home, keep everybody safe!</font></h1>
</div>

<img src="/static/img/home_loris.jpg">

{% endblock %}
