{% extends 'layouts/main.html' %}
{% block title %}Change Your Password{% endblock %}
{% block content %}

<div class="page-header">
    {% from 'macros/render_form.html' import change_render_form %}
    <h2> <a href="{{ url_for('change') }}">Change Your Password</a></h2>
    {{ change_render_form(form) }}
</div>

{% endblock %}
