# python ignore files and folders

ignore_file = (
    ".Python",
    "pip-log.txt",
    "pip-delete-this-directory.txt",
    ".cache",
    ".coverage",
    "local_settings.py",
    "db.sqlite3",
    "db.sqlite3-journal",
    ".webassets-cache",
    "scrapy",
    "ipython_config.py",
    "celerybeat-schedule",
    "celerybeat.pid",
    ".spyderproject",
    ".spyproject",
    ".ropeproject",
    ".dmypy.json",
    "dmypy.json"
)

ignore_file_endings = (
    ".pyc", ".pyo", ".pyd", ".py.class",
    ".so", ".manifest", ".spec",
    ".cover",
    ".pycover",
    ".mo",
    ".pot",
    ".log",
    ".ipynb_checkpoints",
    ".sage.py",
    ".env",
    ".venv"
)

ignore_file_starts = (
    ".coverage.",
)

ignore_folder = (
    "__pycache__",
    "build",
    "develop-egss",
    "dist",
    "downloads",
    "eggs",
    ".eggs",
    "lib",
    "lib64",
    "parts",
    "sdist",
    "var",
    "wheels",
    "python-wheels",
    ".installed.cfg",
    "MANIFEST",
    "htmlcov",
    ".tox",
    ".nox",
    ".coverage",
    ".cache",
    ".hypothesis",
    ".pytest_cache",
    "cover",
    "instance",
    "_build",
    ".pybuilder",
    "target",
    "profile_default",
    "__pypackages__",
    "env",
    "venv",
    "ENV",
    "env.bak",
    "venv.bak",
    "site",
    ".mypy_cache",
    ".pyre",
    ".pytype",
    "cpython_debug"
)

ignore_folder_endings = (
    ".egg-info",
    ".egg"
)

ignore_folder_starts = (
    ".coverage.",
)
