{% extends 'layouts/main.html' %}
{% block title %}Setup Analysis for {{ table_name }}{% endblock %}
{% block content %}

<div class="page-header">
    {% from 'macros/render_form.html' import render_form %}
    <h2> <a href={{url}}>{{table_name}}</a> <h2>
    <h4>Enter Analysis Settings</h4>
    {{ render_form(form) }}
    <hr>
    <h4>Table</h4>
    {% with data=data, toggle_off_keys=toggle_off_keys%}
        {% include 'macros/tables.html' %}
    {% endwith %}
</div>

{% endblock %}
