{% extends 'layouts/main.html' %}
{% block title %}Refreshed Database{% endblock %}
{% block content %}

<div class="page-header">
  <h1>Database has been refreshed!</h1>
</div>
<img src="/static/img/loris_refresh.jpg">

{% endblock %}
