{% extends 'layouts/main.html' %}
{% block title %}Error{% endblock %}
{% block content %}

<div class="page-header">
  <h1>Read error message and notify administators, if necessary!</h1>
</div>
<p>
    <font color="darkred">
        {% for line in traceback%}
            <b><br>{{line}}</br></b>
        {% endfor %}
    </font>
</p>

<img src="/static/img/slow_loris.jpg">

{% endblock %}
